use crate::level::{Artifact, Level, StructureElement};
use crate::player::Player;

pub struct Game {
    pub player: Player,
    pub levels: [Level; 25],
}

impl Game {
    pub fn move_player(&mut self, dx: i16, dy: i16) {
        // check if move is allowed first
        let player_pos = self.player.get_position();
        let level = &self.levels[player_pos.get_level()];
        let new_x: i16 = player_pos.get_x() as i16 + dx;
        let new_y: i16 = player_pos.get_y() as i16 + dy;
        let can_go: bool = match level.get_element(new_x, new_y) {
            (None, _, _) => { false }
            (_, _, Some(e)) => {
                self.levels[player_pos.get_level()].attack(new_x, new_y, &mut self.player)
            }
            (Some(t), _, _) => {
                match t {
                    StructureElement::Wall => { false }
                    _ => { true }
                }
            }
        };
        if can_go {
            let tmp = level.get_element(new_x, new_y);
            match tmp.0.unwrap() {
                StructureElement::StairDown => {
                    let next_level = player_pos.get_level() + 1;
                    let start_x: usize = self.levels[next_level].start_x;
                    let start_y: usize = self.levels[next_level].start_y;
                    self.player.set_position(next_level, start_x, start_y);
                    self.levels[next_level].discover(self.player.get_position());
                }
                StructureElement::StairUp => {
                    self.player.set_position(
                        player_pos.get_level() - 1,
                        self.levels[player_pos.get_level() - 1].end_x,
                        self.levels[player_pos.get_level() - 1].end_y,
                    );
                }
                _ => {
                    self.player.change_position(dx, dy);
                    self.levels[player_pos.get_level()].discover(self.player.get_position());
                }
            };
            match tmp.1 {
                Some(Artifact::Chest {gold}) => {
                    self.levels[player_pos.get_level()].remove_artifact(new_x, new_y);
                    self.player.add_gold(gold);
                }
                _ => {}
            }
            // match tmp.2 {
            //     Some(enemy) => {
            //         // todo!("implement fighting!");
            //         self.levels[player_pos.get_level()].attack(new_x, new_y, &mut self.player);
            //     }
            //     _ => {}
            // }
        }
    }
}