use std::cmp::{max, min};

use crate::position::Position;

pub struct Player {
    name: String,
    position: Position,
    gold: usize,
    life: usize,
    max_life: usize,
}

impl Player {
    pub fn new(name: &str, max_life: usize) -> Player {
        Player {
            name: name.to_string(),
            position: Position::new(0, 0, 0),
            gold: 0,
            life: max_life,
            max_life,
        }
    }
    pub fn get_name(&self) -> String {
        return self.name.clone();
    }
    pub fn get_life(&self) -> usize { self.life }
    pub fn get_max_life(&self) -> usize { self.max_life }
    pub fn get_gold(&self) -> usize { self.gold }
    pub fn add_gold(&mut self, amount: usize) { self.gold += amount; }
    pub fn change_life(&mut self, by: i16) {
        self.life = max(0,
                        min(self.max_life,
                            (self.life as i16 + by) as usize,
                        ),
        );
    }
    pub fn get_position(&self) -> Position {
        self.position
    }
    pub fn change_position(&mut self, dx: i16, dy: i16) {
        self.position.change(dx, dy);
    }

    pub fn set_position(&mut self, level: usize, x: usize, y: usize) {
        self.position.set(level, x, y);
    }
}

#[test]
fn test_get_name() {
    let p = Player {
        name: "Teddy Tester".to_string(),
        position: Position::new(0, 1, 1),
        gold: 0,
        life: 5,
        max_life: 10,
    };
    assert_eq!(p.get_name(), "Teddy Tester");
}

#[test]
fn test_change_life() {
    let mut p = Player {
        name: "Teddy Tester".to_string(),
        position: Position::new(0, 1, 1),
        gold: 0,
        life: 5,
        max_life: 10,
    };
    assert_eq!(p.life, 5);
    p.change_life(-2);
    assert_eq!(p.life, 3);
    p.change_life(10);
    assert_eq!(p.life, 10);
    p.change_life(-12);
    assert_eq!(p.life, 0);
}